/*
 * Simson's XML output class.
 */

#ifndef _XML_H_
#define _XML_H_

#include <string>
#include <stack>
#include <stdio.h>
#include <set>

class xml {
    std::string outfilename;
    FILE  *out;				// where it is being written
    std::set<std::string> tags;			// XML tags
    void  write_doctype(FILE *out);
    void  write_dtd(FILE *out);
    bool  make_dtd;
    std::string  tempfilename;
    void  verify_tag(std::string tag);
    std::stack<std::string>tag_stack;
    void spaces();			// print spaces corresponding to tag stack

public:
    xml();				// private constructor

    void set_outFILE(FILE *out);	  // writes to this FILE without a DTD
    void set_outfilename(std::string outfilename);     // writes to this outfile with a DTD (needs a temp file)
    void set_makeDTD(bool flag);		 // should we write the DTD?

    static std::string xmlescape(const std::string &xml);
    static std::string xmlstrip(const std::string &xml);
    
    void open();			// opens the output file
    void close();			// writes the output to the file
    void tagout(const std::string &tag,const std::string &attribute);
    void xmlout(const std::string &tag,const std::string &value,
		const std::string &attribute,const bool escape_value);
    void xmlout(const std::string &tag,const std::string &value){
	xmlout(tag,value,"",true);
    }
    void xmlprintf(const std::string &tag,const std::string &attribute,
		   const char *fmt,...);
    void push(const std::string &tag,const std::string &attribute);
    void push(const std::string &tag) {push(tag,"");}
    void puts(const std::string &pdata);	// writes a std::string as parsed data
    void printf(const char *fmt,...);	// writes a std::string as parsed data
    void pop();	// close the tag
    void comment(const std::string &comment);
};

#endif
